{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TemplateHaskell #-}

module Main (main) where

import Data.FileEmbed (embedStringFile)
import Data.Function ((&))
import Data.Text.Lazy (Text)
import Network.Wai.Handler.Warp (Port)
import Network.Wai.Middleware.RequestLogger (logStdoutDev)
import Text.Blaze ((!))
import Text.Blaze.Html.Renderer.Text (renderHtml)

import Network.HTTP.Types
import Network.Wai.Middleware.Gzip
import Web.Scotty as S

import qualified Data.List as List
import qualified Data.Maybe as Maybe
import qualified Network.HTTP.Client as Http
import qualified Network.HTTP.Client.TLS as Http


import qualified Control.Exception as E
import qualified Text.Blaze.Html5 as H
import qualified Text.Blaze.Html5.Attributes as A

-- HTML
index :: H.Html -> Text
index body = renderHtml $ do
  H.docTypeHtml $ do
    H.head $ do
      H.title "TESTING Scotty+Elm"
      H.meta ! A.charset "utf-8"
      H.style "body{margin:0px;}"
    (H.body ! A.id "body") $ do
      H.noscript "You don't have Javascript enabled! This site doesn't work without it."
      body

-- CONF
port :: Port
port = 8080

adminContact :: String
adminContact = "[Matrix] @mtgmonkey:calitabby.net"

compiledElmApp :: H.Html
compiledElmApp = H.toHtml ($(embedStringFile "./main.js") :: String)

boilerplateJs :: H.Html
boilerplateJs = H.toHtml ($(embedStringFile "./init.js") :: String)

-- MAIN
main :: IO ()
main = do
  httpman <- Http.newManager Http.tlsManagerSettings
  let tor_ips_req = Http.setQueryString [] "https://raw.githubusercontent.com/7c/torfilter/refs/heads/main/lists/txt/torfilter-2d-flat.txt"
  tor_ips_resp <- Http.httpLbs tor_ips_req httpman
  let tor_ips = show tor_ips_resp
  scotty port $ do
    
    middleware $ gzip $ def { gzipFiles = GzipCompress }
    middleware logStdoutDev
  
    -- GET requests
    get "/" $ do
      client_ip <- header "CLIENT_IP"
      x_forwarded_for <- header "X_FORWARDED_FOR"
      x_forwarded_for_lower <- header "x-forwarded-for"
      x_forwarded <- header "X_FORWARDED"
      x_cluster_client_ip <- header "X_CLUSTER_CLIENT_IP"
      forwarded_for <- header "FORWARDED_FOR"
      forwarded <- header "FORWARDED"
      via <- header "VIA"
      remote_addr <- header "REMOTE_ADDR"
      let ip = [ client_ip
                , x_forwarded_for
                , x_forwarded_for_lower
                , x_forwarded
                , x_cluster_client_ip
                , forwarded_for
                , forwarded
                , via
                , remote_addr
                , Just "NO_IP"
                ]
                & List.dropWhile ((==) Nothing)
                & List.head
                & Maybe.fromMaybe "0.0.0.0"
                & show
                & List.init
                & List.tail
      shortCache
      status ok200
      S.html $ index $ do
        if List.isInfixOf ip tor_ips
        then do
          H.text "You are using Tor. I won't serve Javascript to Tor, because you should have it disabled. Go disable it now and leave :3"
          H.text " (psst. The onionsite lets you use javascript. You can get it from the Onion-Location header)"
        else
          embedJs $ do
            compiledElmApp
            boilerplateJs

    get "/favicon.ico/" $ do
      shortCache
      status notFound404
      S.html $ "you want a favi-<i>what</i>now!?"

    -- ERR
    notFound $ do
      noCache
      status methodNotAllowed405
      S.text "Verb disallowed; OR, route doesn't exist :("


-- FUNC
serverErrorReadFile :: E.IOException -> Js
serverErrorReadFile e = H.toHtml $ "document.getElementById('body').innerHTML='Server-side error occurred: "
                            ++ (show e)
                            ++ "; report this to a site administrator: "
                            ++ adminContact
                            ++ "';"

shortCache :: ActionM ()
shortCache = do
  addHeader "Cache-Control" "max-age=21600"
  addHeader "Onion-Location" "http://mmonkydr7laya5lsrtb4csa5tqlna62cj5qpjuvy726ti4xb3vznneid.onion"

noCache :: ActionM ()
noCache = do
  addHeader "Cache-Control" "no-cache"
  addHeader "Onion-Location" "http://mmonkydr7laya5lsrtb4csa5tqlna62cj5qpjuvy726ti4xb3vznneid.onion"

embedJs :: Js -> H.Html
embedJs js = H.script $ js

 -- TYPES
type AssetPath = FilePath
type Js = H.Html
